package com.sas.drugdev.rapi.permissions;

import java.io.FileOutputStream;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.sas.drugdev.remote.RemoteException;
import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.InvalidTypeException;
import com.sas.drugdev.remote.repository.RemoteNode;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.session.SessionFactory;
import com.sas.drugdev.remote.session.UsernamePasswordCredentials;

public class PermissionsReport {
	private Session session;
	private RepositoryService service;
	private List entries;
	private boolean verbose = false;
	
	public PermissionsReport(String url, String username, String password){
		try{
			session = SessionFactory.newSession(new URL(url), new UsernamePasswordCredentials(username, password.toCharArray()));
			service = session.getServiceManager().getRepositoryService();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void constructReport(String SDDPath, String savePath, int levels){
		try{
			
			FileOutputStream outfile = new FileOutputStream(savePath);
			//outfile.write(",,Permission,,,,N\n".getBytes());
			outfile.write("Path,Owner,Re,Wr,De,Ma,Num Files,Num Folders,Group,User\n".getBytes());
			
			entries = collectContainerEntries(SDDPath, levels);
			
			for(Iterator it = entries.iterator(); it.hasNext(); ){
				ContainerEntry entry = (ContainerEntry)it.next();
				
				String output = "";// entry.getPath() + "," + entry.getOwner() + ",";
				
				//boolean beenRun = false;
				for(Iterator iter = entry.getEntityACEs().keySet().iterator(); iter.hasNext(); ){					
					String code = (String)iter.next();
					UGEntity entity = (UGEntity)entry.getEntityACEs().get(code);
					if(entity.size() > 0){
						//properly format the output string
						//if(beenRun)
							//output = ",,";
							output = entry.getPath() + "," + entry.getOwner() + ",";
						//else
							//beenRun = true;
						
						//parse the permissions code
						for(int i = 0; i < code.length(); i++){
							if(code.charAt(i) == 'y')
								output += "Y,";
							else output += "N,";
						}
						
						//output the number of files and folders
						output += entry.getNumFiles() + ",";
						output += entry.getNumFolders() + ",";
						
						//collect all the groups
						for(int i = 0; i < entity.getGroups().size(); i++){
							if(entity.getGroups().get(i).equals("AUTHENTICATED@PUBLIC"))
								output += "Users With Accounts :: ";
							else
								output += entity.getGroups().get(i) + " :: ";
						}
						if(output.endsWith(":: "))
							output = output.substring(0, output.length()-3);
						output += ",";
						
						//collect all the users
						for(int i = 0; i < entity.getUsers().size(); i++){
							if(entity.getUsers().get(i).equals("OWNER@PUBLIC"))
								output += "Owner :: ";
							else
								output += entity.getUsers().get(i) + " :: ";
						}
						if(output.endsWith(":: "))
							output = output.substring(0, output.length()-3);
						output += "\n";
						outfile.write(output.getBytes());
					}
				}
			}
			outfile.close();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	//called to begin the recursive process
	public List collectContainerEntries(String SDDPath, int levels) throws Exception{
		List ret = new ArrayList();
		
		collectContainerEntry(ret, service.getNode(new ContainerBean(SDDPath)), levels);
		
		return ret;
	}
	
	//recursively gathers the children and populates a list with all their information
	private List collectContainerEntry(List entries, RemoteNode node, int levels) throws Exception{
		if(levels < 0)
			return entries;
		
		if(verbose)
			System.out.println(node.getPath());

		
		ContainerEntry entry = new ContainerEntry(node.getPath());
		entry.setOwner(node.getOwner());
		
		try{
			entry.addACL(service.getAccessControlList(new ContainerBean(node.getPath())));
		}
		catch(RemoteException e){
			return entries;
		}
		
		List children = null;
		try{
			children = service.getChildren(new ContainerBean(node.getPath()));
		}
		catch(InvalidTypeException e){
			entries.add(entry);
			return entries;
		}
		
		//count the number of files and folders
		int numFiles = 0, numFolders = 0;
		for(Iterator iter = children.iterator(); iter.hasNext(); ){
			RemoteNode remoteNode = (RemoteNode)iter.next();
			if(remoteNode.getType().equals("folder")){
				numFolders++;
			}
			else numFiles++;
		}
		entry.setNumFiles(numFiles);
		entry.setNumFolders(numFolders);
		
		entries.add(entry);
		
		for(Iterator iter = children.iterator(); iter.hasNext(); ){
			RemoteNode remoteNode = (RemoteNode)iter.next();
			collectContainerEntry(entries, remoteNode, levels-1);
		}
		
		return entries;
	}
	
	private static void usage(){
		System.out.println("Usage:");
		System.out.println("java[w] -jar PermissionsReport.jar <SDD url> <username> <password> <SDD path> <depth> <save file path> <start time> [-v]");
		System.out.println("-SDD url: URL to your SDD instance's remote API eg. https://sddyourinstance.sas.com/sddremote");
		System.out.println("-username: username to login as");
		System.out.println("-password: password to login with");
		System.out.println("-SDD path: head node the permission report will start at");
		System.out.println("-depth: how many levels to process");
		System.out.println("-Save file path: path and filename of the output file eg. C:\\permissions\\report.csv");
		System.out.println("-Start time: time of day and day to run the report. Format(Military) = MMDDYYYY:HHMM (eg 12252008:2200");
		System.out.println("\tTo run immediately, use the parameter 'now' or 'NOW' for start time");
		System.out.println("-v: Verbose - optional parameter - shows the progress of the job by outputting the folder currently being processed");
		System.exit(1);
	}
	
	private static void schedule(Date start){
		try{
			Date current = new Date();
			while(start.compareTo(current) > 0){
				Thread.sleep(10000);
				current = new Date();
			}
		}
		catch(Exception e){
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void setVerbose(boolean verbose){
		this.verbose = verbose;
	}
	
	public static void main(String [] args){
		if(args.length < 7)
			usage();
			
		
		String url = args[0];
		String username = args[1];
		String password = args[2];
		String SDDPath = args[3];
		int depth = Integer.parseInt(args[4]);
		String savePath = args[5];
		String startTime = args[6];			
		
		Date start = new Date();
		
		if( !(startTime.equals("now") || startTime.equals("NOW") || startTime.equals("Now")) ){
			DateFormat format = new SimpleDateFormat("MMddyyyy:HHmm");

			try{
				start = format.parse(startTime);
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
			System.out.println("Process Scheduled to begin at " + start);
			PermissionsReport.schedule(start);
		}
		
		PermissionsReport report = new PermissionsReport(url, username, password);
		if(args.length > 7)
			report.setVerbose(true);
		start = new Date();
		System.out.println("Process Started at " + start);
		report.constructReport(SDDPath, savePath, depth);
		Date finish = new Date();
		
		System.out.println("Process Completed at " + finish);
	}
}
